<?php
/**
 * This file is part of the JustKnit package.
 *
 * @copyright JustKnit (Iris Hammer) <iris@justknit.de>
 * @license All rights reserved (@https://dejure.org/gesetze/UrhG/69a.html)
 * Created on 03-20-2023
 *
 * @package justknit
 */

namespace JustKnit\Calculate;

use MODX\Revolution\modX;
use MODX\Revolution\modChunk;
use JustKnit\Calculate\Dictionary;
use JustKnit\Calculate\Hooks;
use JustKnit\Calculate\Validator;

/**
 * Main Calculate class.
 *
 * @package justknit
 * @subpackage Calculate
 */

class Calculate {
    /**
     * The constructor for the Calculate class.
     *
     * @param modX &$modx A reference to the modX instance
     * @param array $config An array of configuration properties
     */
    public function __construct(modX &$modx, array $config = []) {
        $this->modx = &$modx;
        $corePath = $this->modx->getOption(
            'justknit.core_path',
            $config,
            $this->modx->getOption(
                'core_path',
                null,
                MODX_CORE_PATH
            ) . 'components/justknit/'
        );
        $this->config = array_merge([
            'corePath'        => $corePath,
            'modelPath'       => $corePath . 'src/Model/',
            'controllersPath' => $corePath . 'src/Controllers/Calculate/',
            'processorsPath'  => $corePath . 'src/Processors/Calculate/',
            'chunksPath'      => $corePath . 'elements/chunks/',
        ], $config);
        $this->modx->lexicon->load('justknit:frontend');
    }
    /**
     * Load a Calculate controller by a given name.
     * (used by Snippets)
     *
     * @param string $controller
     * @return Controller object|null
     */
    public function loadController(string $controller) {
        $classPath = $this->config['controllersPath'] . $controller . '.php';
        $className = 'JustKnit\\Controllers\\Calculate\\' . $controller;
        if (file_exists($classPath)) {
            if (!class_exists($className)) {
                $className = require_once $classPath;
            }
            if (class_exists($className)) {
                $this->controller = new $className($this, $this->config);
            } else {
                $this->modx->log(
                    modX::LOG_LEVEL_ERROR,
                    '[JustKnit] Could not load controller ' . $className . ' at ' . $classPath
                );
            }
        } else {
            $this->modx->log(modX::LOG_LEVEL_ERROR, '[JustKnit] Could not find controller file: ' . $classPath);
        }
        return $this->controller;
    }
    /**
     * Load the Dictionary class and gather $_POST and $_GET params.
     *
     * @access public
     * @return Dictionary An instance of the Dictionary class
     */
    public function loadDictionary() {
        $this->dictionary = new Dictionary($this);
        // Load POST and GET parameters
        $this->dictionary->gather();
        return $this->dictionary;
    }
    /**
     * Load the Validator class and gather $_POST and $_GET params.
     *
     * @access public
     * @return Validator An instance of the Validator class
     */
    public function loadValidator() {
        $this->validator = new Validator($this);
        // Load POST and GET parameters
        return $this->validator;
    }    /**
     * Loads the Hooks class.
     *
     * @access public
     * @param string $type The name of the Hooks service to load (preHooks, postHooks)
     * @param array $config array An array of configuration parameters for the hooks class
     * @return Hooks An instance of the Hooks class
     */
    public function loadHooks(string $type, array $config = []) {
        $this->$type = new Hooks($this, $config);
        return $this->$type;
    }
}
