<?php
/**
 * This file is part of the JustKnit package.
 *
 * @copyright JustKnit (Iris Hammer) <iris@justknit.de>
 * @license All rights reserved (@https://dejure.org/gesetze/UrhG/69a.html)
 * Created on 03-20-2023
 *
 * @package justknit
 */

namespace JustKnit\Controllers\Calculate;

use MODX\Revolution\modX;
use MODX\Revolution\modUser;
use MODX\Revolution\modUserProfile;
use MODX\Revolution\modUserGroup;
//use MODX\Revolution\modResource;
use JustKnit\Model\JustKnitClientMeta;
use JustKnit\Model\JustKnitCalc;
use JustKnit\Model\JustKnitDim;
use JustKnit\Calculate\Calculate;
/**
 * Base controller class for handling of calculation.
 *
 * @package justknit
 * @subpackage controllers
 */
abstract class Base {
    /** @var array $config An array of configuration properties */
    public $config = [];
    /** @var array $scriptProperties */
    public $scriptProperties = [];
    /** @var array $fields An array of key->name pairs storing the fields passed */
    public $fields = [];
    /**
     * The constructor for the Base controller class.
     *
     * @param alculate $calculate A reference to the alculate instance
     * @param array $config
     */
    public function __construct(Calculate &$calculate, array $config = []) {
        $this->modx = &$calculate->modx;
        $this->calculate = &$calculate;
        $this->config = array_merge($this->config, $config);
        $this->modx->lexicon->load('justknit:frontend');
        $this->modx->lexicon->load('justknit:calc');
    }
    /**
     * Run a process.
     *
     * @param array $scriptProperties An array of properties
     * @access public
     */
    public function run(array $scriptProperties) {
        $this->setProperties($scriptProperties);
        $this->initialize();
        return $this->process();
    }
    /**
     * Abstracts initialize method.
     */
    abstract public function initialize();
    /**
     * Abstracts process method.
     */
    abstract public function process();
    /**
     * Gets a user object by the MODX user ID.
     *
     * @access public
     * @params integer $userID
     * @return modUser object or null
     */
    public function getUserById(int $userID) {
        $this->user = $this->modx->getObject(modUser::class, [
            'id' => $userID,
        ]);
        if (!is_object($this->user)) {
            $this->user = null;
            $this->modx->log(modX::LOG_LEVEL_INFO, '[JustKnit] Could not load user with id: ' . $userID);
        }
        return $this->user;
    }
    /**
     * Get the Profile of the active user object.
     *
     * @access public
     * @return modUserProfile object or null
     */
    public function getProfile() {
        if (!is_object($this->user)) {
            $this->modx->log(modX::LOG_LEVEL_ERROR, '[JustKnit] User object to load profile doesn\'t exist.');
            return false;
        }
        $this->profile = $this->user->getOne('Profile');
        if (!is_object($this->profile)) {
            $this->profile = null;
            $this->modx->log(modX::LOG_LEVEL_INFO,'[JustKnit] Could not load profile for user: ' . $this->user->get('username'));
        }
        return $this->profile;
    }
    /**
     * Get ClientMeta of the active user object.
     *
     * @access public
     * @return JustKnitClientMeta object or null
     */
    public function getClientMeta() {
        if (!is_object($this->user)) {
            $this->modx->log(modX::LOG_LEVEL_ERROR, '[JustKnit] User object to load ClientMeta doesn\'t exist.');
            return false;
        }
        $this->clientmeta = $this->modx->getObject(JustKnitClientMeta::class, [
            'client_id' => $this->user->get('id'),
        ]);
        if (!is_object($this->clientmeta)) {
            $this->clientmeta = null;
            $this->modx->log(modX::LOG_LEVEL_INFO,'[JustKnit] Could not load JustKnitClientMeta for user with id: ' . $this->user->get('id'));
        }
        return $this->clientmeta;
    }
    /**
     * Get name of the primary_group of active user.
     *
     * @access public
     * @return string
     */
    public function getMainGroup() {
        $userGroup = $this->user->get('primary_group');
        $this->primarygroup = $this->modx->getObject(modUserGroup::class, [
            'id' => $userGroup,
        ]);
        $this->mainGroup = $this->primarygroup->get('name');
        return $this->mainGroup;
    }
    /**
     * Get JustKnitCalc for active User and requested ressource
     *
     * @access public
     * @return JustKnitCalc object or null
     */
    public function getCalc() {
        if (!is_object($this->user)) {
            $this->modx->log(modX::LOG_LEVEL_ERROR, '[JustKnit] User object to load JustKnitCalc doesn\'t exist.');
            return false;
        }
        $this->calc = $this->modx->getObject(JustKnitCalc::class, [
            'ccreatedby' => $this->user->get('id'),
            'resource_id' => $this->modx->resource->get('id'),
        ]);
        if (!$this->calc) {
            $this->calc = null;
        }
        return $this->calc;
    }
    /**
     * Get JustKnitDim for active User, requested ressource and requested Calc
     *
     * @access public
     * @return array of JustKnitDim objects or null
     */
    public function getDims(int $calcId) {
        if (!is_object($this->user)) {
            $this->modx->log(modX::LOG_LEVEL_ERROR, '[JustKnit] User object to load JustKnitCalc doesn\'t exist.');
            return false;
        }
        $this->dims = $this->modx->getCollection(JustKnitDim::class, [
            'calcedby' => $this->user->get('id'),
            'resource_id' => $this->modx->resource->get('id'),
            'calc_id' => $calcId,
        ]);
        if (!$this->dims) {
            $this->dims = null;
        }
        return $this->dims;
    }
    /**
     * Return an array of REQUEST script properties.
     *
     * @access public
     * @return array
     */
    public function getProperties() {
        return $this->scriptProperties;
    }
    /**
     * Check if $_POST exists + load and set dictionary.
     *
     * @access public
     * @return boolean
     */
    public function hasPost() {
        $hasPost = false;
        if (!empty($_POST)) {
            $this->dictionary = $this->calculate->loadDictionary();
            if ($this->dictionary) {
                $hasPost = true;
            }
        }
        return $hasPost;
    }
    /**
     * Run a desired processor.
     *
     * @access public
     * @param string $processor Name of the processor to run
     * @return mixed|string
     */
    public function runProcessor(string $processor) {
        $output = '';
        $processor = $this->loadProcessor($processor);
        if (empty($processor)) {
            return $output;
        }
        // Return the output of the processor
        return $processor->process();
    }
    /**
     * Loads a processor.
     *
     * @access public
     * @param string $processor Name of the processor to load
     * @return bool|Subscription processor
     */
    public function loadProcessor(string $processor) {
        $processorFile = $this->config['processorsPath'] . $processor . '.php';
        if (!file_exists($processorFile)) {
            $this->modx->log(modX::LOG_LEVEL_ERROR, '[JustKnit] Could not find processor file: ' . $processorFile);
            return false;
        }
        try {
            $className = 'JustKnit\\Processors\\Calculate\\' . $processor;
            if (!class_exists($className)) {
                $className = include_once $processorFile;
            }
            $processor = new $className($this->calculate, $this);
        } catch (\Exception $e) {
            $processor = false;
            $this->modx->log(modX::LOG_LEVEL_ERROR, '[JustKnit] ' . $e->getMessage());
        }
        return $processor;
    }
    /**
     * Set the default script properties.
     *
     * @param array $defaults
     * @return void
     */
    function setDefaultProperties(array $defaults = []) {
        $this->scriptProperties = array_merge($defaults, $this->scriptProperties);
    }
    /**
     * Set an array of script properties.
     *
     * @access public
     * @param array $array
     * @return void
     */
    public function setProperties(array $array) {
        foreach ($array as $k => $v) {
            $this->setProperty($k, $v);
        }
    }
    /**
     * Set a script property.
     *
     * @access public
     * @param string $key
     * @param mixed $value
     * @return void
     */
    public function setProperty(string $key, $value) {
        $this->scriptProperties[$key] = $value;
    }
    /**
     * Get a script property.
     *
     * @access public
     * @param string $key
     * @param mixed $default
     * @param string $method
     * @return mixed
     */
    public function getProperty(string $key, $default = null, string $method = '!empty') {
        $v = $default;
        switch ($method) {
            case 'empty':
            case '!empty':
                if (!empty($this->scriptProperties[$key])) {
                    $v = $this->scriptProperties[$key];
                }
                break;
            case 'isset':
            default:
                if (isset($this->scriptProperties[$key])) {
                    $v = $this->scriptProperties[$key];
                }
                break;
        }
        return $v;
    }
    /**
     * Gets a Chunk and caches it; also falls back to file-based templates
     * for easier debugging.
     *
     * Will always use the file-based chunk if $debug is set to true.
     *
     * @access public
     * @param string $name The name of the Chunk
     * @param array $properties The properties for the Chunk
     * @return string The processed content of the Chunk
     */
    public function getChunk($name, $properties = []) {
        if ($this->modx->services->has('pdoTools')) {
            $pdo = $this->modx->services->get('pdoTools');

            return $pdo->getChunk($name, $properties);
        }
        $chunk = null;
        if (substr($name, 0, 6) === '@CODE:') {
            $content = substr($name, 6);
            $chunk = $this->modx->newObject(modChunk::class);
            $chunk->setContent($content);
        } elseif (!isset($this->chunks[$name])) {
            if (!$this->config['debug']) {
                $chunk = $this->modx->getObject(modChunk::class, ['name' => $name], true);
            }
            if (empty($chunk)) {
                $chunk = $this->_getTplChunk($name);
                if ($chunk === false) {
                    return false;
                }
            }
            $this->chunks[$name] = $chunk->getContent();
        } else {
            $content = $this->chunks[$name];
            $chunk   = $this->modx->newObject(modChunk::class);
            $chunk->setContent($content);
        }
        $chunk->setCacheable(false);
        return $chunk->process($properties);
    }
}
