<?php
/**
 * This file is part of the JustKnit package.
 *
 * @copyright JustKnit (Iris Hammer) <iris@justknit.de>
 * @license All rights reserved (@https://dejure.org/gesetze/UrhG/69a.html)
 * Created on 03-20-2023
 *
 * @package justknit
 */

namespace JustKnit\Controllers\Register;

use MODX\Revolution\modX;
use MODX\Revolution\modUser;
use MODX\Revolution\modUserProfile;
use JustKnit\Model\JustKnitClientMeta;
use JustKnit\Model\JustKnitProduct;
use JustKnit\Model\JustKnitOrder;
use JustKnit\Controllers\Register\Base;

class RegisterDS24 extends Base {

    /** @var array $post An array of posted values from DS24 */
    public $post = [];
    /**
     * Load default properties for this controller.
     *
     * @return void
     */
    public function initialize()
    {
        $this->setDefaultProperties([
            'toJson'         => true,
            'DS24HeadTpl'    => 'JustKnit.DS24HeadChunk',
            'DS24KeyTpl'     => 'JustKnit.DS24KeyChunk',
            'DS24HeadRtnTpl' => 'JustKnit.DS24HeadRtnChunk',
            'DS24KeyRtnTpl'  => 'JustKnit.DS24KeyRtnChunk',
            'primary_group'  => false,
            'blocked'        => false,
            'active'         => true,
            'client_deleted' => false,
        ]);
    }

    /**
     * Handle the JustKnit Register logic.
     *
     * @return string
     */
    public function process()
    {
        $userID = false;
        if (!$this->hasPost()) {
            return $this->getJsonError('no valid $Post');
        }
        $this->validateSha();
        if (!$this->shaIsValid) {
            return $this->getJsonError('invalid sha signature');
        }
        // get product and set grop and duration values
        $productId = $this->getProductId();
        if (!$productId){
            return $this->getJsonError('no valid product_id in $_POST');
        }
        if ($this->getProduct()) {
            $this->setProperty('productID', $this->product->get('id'));
            $this->setProperty('primary_group', $this->product->get('modxusergroup'));
            $this->setProperty('duration', $this->product->get('duration'));
            $this->setDurationValues();
        }
        if (!$this->getProperty('primary_group', false)) {
            return $this->getJsonError('no valid modUserGroup found for product ' . $productId);
        }
        // user is identified via the email
        // allow_multiple_emails = false
        $userEmail = $this->getProperty('email', false);
        if (!$userEmail){
            return $this->getJsonError('no valid email in $_POST');
        }
        $userID = $this->getUserByEmail($userEmail);
        if ($userID){
            $existingUser = true;
            if ($this->getUserById($userID)) {
                if (!$this->getProfile()) {
                    return $this->getJsonError('no profile for User with ID ' . $userID);
                }
            }
            // existing JustKnit Client
            if ($this->getClientMetaByUserId($userID)) {
                // Update existing modUser, modProfile, JustKnitClientMeta
                $result = $this->runProcessor('UpdateClient');
                if (!$result) {
                    return $this->getJsonError('Client data could not be saved');
                }
            }
        } else {
            $existingUser = false;
            $this->trimAddress();
            $this->mapNewUserFields();
            $result = $this->runProcessor('RegisterClient');
            if (!$result) {
                return $this->getJsonError('New user could not be created');
            }
        }
        $keydata = $this->setKeyData($existingUser);
        return $this->getJsonSuccess();
    }
    /**
     * Set success message for JsonResponse
     *
     * @access public
     * @return bool
     */
    public function setKeyData($newUser)
    {
        $properties = $this->getProperties();
        if ($newUser) {
            $headChunk = $this->getProperty('DS24HeadRtnTpl','DS24HeadRtnTpl');
            $keyChunk = $this->getProperty('DS24KeyRtnTpl','DS24KeyRtnTpl');
        } else {
            $headChunk = $this->getProperty('DS24HeadTpl','DS24HeadTpl');
            $keyChunk = $this->getProperty('DS24KeyTpl','DS24KeyTpl');
        }
        $head = $this->modx->getChunk($headChunk, $properties);
        $key = $this->modx->getChunk($keyChunk, $properties);
        $data = array(
                'fullname' => $properties['first_name'].' '.$properties['last_name'],
                'order_id' => $properties['order_id'],
                'product_name' => $properties['product_name'],
                'AboEnde'  => strftime('%c',$properties['blockedafter']),
            );
        $this->response = array(
                'status' => 'success',
                'key' => $key,
                'data'   => $data,
                'headline' => $head,
        );
        return true;
    }
}

