<?php
/**
 * This file is part of the JustKnit package.
 *
 * @copyright JustKnit (Iris Hammer) <iris@justknit.de>
 * @license All rights reserved (@https://dejure.org/gesetze/UrhG/69a.html)
 * Created on 03-20-2023
 *
 * @package justknit
 */

namespace JustKnit;

use MODX\Revolution\modX;
use MODX\Revolution\modUser;
use MODX\Revolution\modProfile;
use MODX\Revolution\modChunk;
use MODX\Revolution\Transport\modTransportPackage;
use MODX\Revolution\modNamespace;
use JustKnit\Model;

/**
 * JustKnit main class
 *
 * @package justknit
 */
 class JustKnit {
    /** @var $modx \MODX\Revolution\modX reference to the modX object */
    public $modx = null;

    /** @var array $config An array of configuration properties */
    public $config= [];

    /** @var boolean $debug Debug mode on/off */
    public $debug = false;

    /**
     * The constructor for the JustKnit class.
     *
     * @param \MODX\Revolution\modX &$modx A reference to the modX object
     * @param array $config An array of configuration properties
     */
    function __construct(modX &$modx, array $config = [])
    {
        $this->modx =& $modx;
        $corePath = $this->modx->getOption(
            'justknit.core_path',
            $config,
            $this->modx->getOption('core_path', null, MODX_CORE_PATH).'components/justknit/');
        $assetsPath = $this->modx->getOption(
            'justknit.assets_path',
            $config,
            $this->modx->getOption('assets_path', null, MODX_ASSETS_PATH).'components/justknit/');
        $assetsUrl = $this->modx->getOption(
            'justknit.assets_url',
            $config,
            $this->modx->getOption('assets_url', null, MODX_ASSETS_URL) . 'components/justknit/'
        );
        $this->config = array_merge([
            'corePath'       => $corePath,
            'srcPath'        => $corePath . 'src/',
            'modelPath'      => $corePath . 'src/Model/',
            'processorsPath' => $corePath . 'src/Processors/',
            'chunksPath'     => $corePath . 'elements/chunks/',
            'docsPath'       => $corePath . 'docs/',
            'assetsUrl'      => $assetsUrl,
            'assetsPath'     => $assetsPath,
            'jsUrl'          => $assetsUrl . 'js/',
            'cssUrl'         => $assetsUrl . 'css/',
            'imgUrl'         => $assetsUrl . 'img/',
        ],$config);

        $this->modx->lexicon->load('justknit:default');
    }
    /**
     * Gets a Chunk and caches it + fall back to file-based templates for easier debugging.
     *
     * @access public
     * @param string $name The name of the Chunk
     * @param array $properties The properties for the Chunk
     * @return string The processed content of the Chunk
     */
    public function getChunk($name, array $properties = [])
    {
/*        $output = '';
        $output = $this->modx->getChunk($name, $properties);
        return $output;
*/

        if (class_exists('pdoTools') && $pdo = $this->modx->getService('pdoTools')) {
            return $pdo->getChunk($name, $properties);
        }

        $chunk = null;

/**
 * TODO:
 * Alternativ: function parseTpl() siehe
 * core/components/goodnews/src/GoodNews.php:680
 */

        if (substr($name, 0, 6) === '@CODE:') {
            $content = substr($name, 6);
            $chunk = $this->modx->newObject(modChunk::class);
            $chunk->setContent($content);
        } elseif (!isset($this->chunks[$name])) {
            $this->modx->log( modX::LOG_LEVEL_ERROR,'[JustKnit] $chunk '.$name .' '. print_r($properties, 1));
            $chunk = $this->_getTplChunk($name);
            if (empty($chunk)) {
                $chunk = $this->modx->getObject(modChunk::class, ['name' => $name]);
                if ($chunk == false) {
                    return false;
                }
            }
            $this->chunks[$name] = $chunk->getContent();
        } else {
            $o = $this->chunks[$name];
            $chunk = $this->modx->newObject(modChunk::class);
            $chunk->setContent($o);
        }
        $chunk->setCacheable(false);
        return $chunk->process($properties);

    }

    /**
     * Returns a modChunk object from a template file.
     *
     * @access private
     * @param string $name The name of the Chunk. Will parse to name.$postfix
     * @param string $postfix The default postfix to search for chunks at.
     * @return modChunk/boolean Returns the modChunk object if found, otherwise false.
     */
    private function _getTplChunk($name, $postfix = '.chunk.tpl')
    {
        $chunk = false;
        $f = $this->config['chunksPath'] . strtolower($name) . $postfix;
        if (file_exists($f)) {
            $o = file_get_contents($f);
            $chunk = $this->modx->newObject(modChunk::class);
            $chunk->set('name', $name);
            $chunk->setContent($o);
        }
        return $chunk;
    }

}
