<?php
/**
 * This file is part of the JustKnit package.
 *
 * @copyright JustKnit (Iris Hammer) <iris@justknit.de>
 * @license All rights reserved (@https://dejure.org/gesetze/UrhG/69a.html)
 * Created on 03-20-2023
 *
 * @package justknit
 */
namespace JustKnit\Processors\Register;

use MODX\Revolution\modX;
use MODX\Revolution\modUser;
use MODX\Revolution\modUserProfile;
use MODX\Revolution\modUserGroup;
use MODX\Revolution\modUserGroupMember;
use JustKnit\Model\JustKnitClientMeta;
use JustKnit\Model\JustKnitProduct;
use JustKnit\Model\JustKnitOrder;
use JustKnit\Processors\Register\Base;

/**
 * Processor class which handles registration when Client with
 * posted [email] already exists
 *  - no new MODX user is created
 *  - active order, active product, client_deletedafter,
 *    client_deleted is updated, [JustKnitClientMeta]
 *  - active, primary_group updated [modUser]
 *  - phone, blocked, blockedafter, all address data updated [modUserProfile]
 *
 * @package justknit
 * @subpackage processors
 */
class ClientUpdateProfile extends Base
{
    /** @var modX $modx A reference to the modX instance */
    public $modx = null;

    /** @var modUser $user */
    public $user = null;

    /** @var modUserProfile $profile */
    public $profile = null;

    /** @var JustKnitClientMeta $clientmeta */
    public $clientmeta = null;

    /** @var boolean $usernameChanged */
    public $usernameChanged = false;
    /** @var string $oldUsername */
    public $oldUsername;
    /**
     * @access public
     * @return mixed
     */
    public function process()
    {
        $this->user  = $this->controller->user;
        $this->profile = $this->controller->profile;
        $this->clientmeta = $this->controller->clientmeta;
        if (empty($this->profile)) {
            return $this->modx->lexicon('justknit.profile_err_nf');
        }
        $this->removeSpamFields();
        $this->setUserFields();
        if (!$this->user->save()) {
            $this->modx->log(modX::LOG_LEVEL_ERROR, '[JustKnit] Could not save client user data - ' . $this->user->get('id') . ' with username: ' . $this->user->get('username'));
            $this->controller->setResponseMessage('Could not save client user data ' . $this->user->get('id'));
            return $this->modx->lexicon('justknit.user_err_save');
        }
        if (!$this->clientmeta->save()) {
            $this->modx->log(modX::LOG_LEVEL_ERROR, '[JustKnit] Could not save client user data - ' . $this->user->get('id') . ' with username: ' . $this->user->get('username'));
            $this->controller->setResponseMessage('Could not save client user data ' . $this->user->get('id'));
            return $this->modx->lexicon('justknit.user_err_save');
        }
        return true;
    }

    /**
     * Remove any spam/submitVar fields from the field list
     * @return void
     */
    public function removeSpamFields() {
        $this->controller->dictionary->remove('nospam');
        $this->controller->dictionary->remove('blank');
        $this->controller->dictionary->remove('email');
        $submitVar = $this->controller->getProperty('submitVar');
        if (!empty($submitVar)) {
            $this->controller->dictionary->remove($submitVar);
        }
    }
    /**
     * Set the user data and create the user/profile objects.
     *
     * @access public
     * @return void
     */
    public function setUserFields() {
        $fields = $this->controller->dictionary->toArray();

//$this->modx->log(modX::LOG_LEVEL_ERROR, '[JustKnit] $fields - unsetUserFields()' . print_r($fields,1));
        foreach ($fields as $k => $v) {
            $this->user->set($k, $v);
            $this->profile->set($k, $v);
            $this->clientmeta->set($k, $v);
        }
        return true;
    }

}
