<?php
/**
 * This file is part of the JustKnit package.
 *
 * @copyright JustKnit (Iris Hammer) <iris@justknit.de>
 * @license All rights reserved (@https://dejure.org/gesetze/UrhG/69a.html)
 * Created on 03-20-2023
 *
 * @package justknit
 */
namespace JustKnit\Processors\Register;

use MODX\Revolution\modX;
use MODX\Revolution\modUser;
use MODX\Revolution\modUserProfile;
use MODX\Revolution\modUserGroup;
use MODX\Revolution\modUserGroupMember;
use JustKnit\Model\JustKnitClientMeta;
use JustKnit\Model\JustKnitProduct;
use JustKnit\Model\JustKnitOrder;
use JustKnit\Processors\Register\Base;

/**
 * Processor class which handles registration when posted [email] not exists
 *  - new MODX user is created
 *  - [modUserProfile] and [JustKnitClientMeta] is filled with data
 *
 * @package justknit
 * @subpackage processors
 */
class RegisterClient extends Base
{
    /** @var modUser $user */
    public $user = null;

    /** @var modUserProfile $profile */
    public $profile = null;

    /** @var JustKnitClientMeta $clientmeta */
    public $clientmeta = null;
    /**
     * @access public
     * @return mixed
     */
    public function process()
    {
        $this->user = $this->modx->newObject(modUser::class);
        $this->profile = $this->modx->newObject(modUserProfile::class);
        $this->clientmeta = $this->modx->newObject(JustKnitClientMeta::class);
        $this->order = $this->modx->newObject(JustKnitOrder::class);
        // Save user
        $this->setUserFields();
        if (!$this->user->save()) {
            $this->modx->log(modX::LOG_LEVEL_ERROR, '[JustKnit] Could not save new client user data - ' . $this->user->get('id') . ' with username: ' . $this->user->get('username'));
            $this->controller->setResponseMessage('Could not save new client user data ' . $this->user->get('id'));
            return $this->modx->lexicon('justknit.user_err_save');
        } else {
            $this->user->joinGroup($this->user->get('primary_group'),'Member');
        }
        //Save order
        $this->setOrderFields();
        if (!$this->order->save()) {
            $this->modx->log(modX::LOG_LEVEL_ERROR, '[JustKnit] Could not save new order data - ' . $this->order->get('id') . ' with ds24_order_id: ' . $this->order->get('order_id'));
            $this->controller->setResponseMessage('[JustKnit] Could not save new order data - ' . $this->order->get('id') . ' with ds24_order_id: ' . $this->order->get('order_id'));
            return $this->modx->lexicon('justknit.user_err_save');
        }
        //Save ClientMeta
        $this->setClientMetaFields();
        if (!$this->clientmeta->save()) {
            $this->modx->log( modX::LOG_LEVEL_ERROR, '[JustKnit] Could not save new clientmeta data - ' . $this->clientmeta->get('id') . ' with ds24_order_id: ' . $this->clientmeta->get('buyer_id'));
            $this->controller->setResponseMessage('[JustKnit] Could not save new clientmeta data - ' . $this->clientmeta->get('id') . ' with buyer_id: ' . $this->clientmeta->get('buyer_id'));
            return $this->modx->lexicon('justknit.user_err_save');
        }
        return true;
    }

    /**
     * Set the user data and create the user/profile objects.
     *
     * @access public
     * @return void
     */
    public function setUserFields()
    {
        $fields = $this->dictionary->toArray();
        $fields = array_merge($fields, $this->controller->getProperties());
        $newPw = $this->_generatePassword();
        $fields = array_merge([
            //modUser
            'password'  => $newPw,
            'active'    => 1,
            //modUserProfile
            'blocked'   => 0,
            'comment'   => ' PW: ' . $newPw,
            'class_key' => 'MODX\\Revolution\\modUser',
            // key to output password in chunk
            'pwd'       => $newPw,
        ], $fields);
        $this->controller->setProperties($fields);
        $this->user->fromArray($fields);
        $this->profile->fromArray($fields);
        $this->user->addOne($this->profile, 'Profile');
        // user->joinGroup not possible for new Users
        return true;
    }
    /**
     * generate new Passord.
     *
     * @access private
     * @return boolean $success
     */
    private function _generatePassword() {
        $password = $this->user->generatePassword();
        return $password;
    }

    /**
     * Set the Order data.
     *
     * @access public
     * @return void
     */
    public function setOrderFields()
    {
        $this->controller->setProperty('client_id' , $this->user->get('id'));
        $this->controller->setProperty('product_id' , $this->controller->getProperty('productID'));
        $this->order->fromArray($this->controller->getProperties());
    }
    /**
     * Set the ClientMeta data.
     *
     * @access public
     * @return void
     */
    public function setClientMetaFields()
    {
        $orderid = $this->order->get('id');
        $this->controller->setProperty('active_order', $orderid);
        $this->clientmeta->fromArray($this->controller->getProperties());
    }
}

