<?php
/**
 * This file is part of the JustKnit package.
 *
 * @copyright JustKnit (Iris Hammer) <iris@justknit.de>
 * @license All rights reserved (@https://dejure.org/gesetze/UrhG/69a.html)
 * Created on 03-20-2023
 *
 * @package justknit
 */
namespace JustKnit\Processors\Register;

use MODX\Revolution\modX;
use MODX\Revolution\modUser;
use MODX\Revolution\modUserProfile;
use MODX\Revolution\modUserGroup;
use MODX\Revolution\modUserGroupMember;
use JustKnit\Model\JustKnitClientMeta;
use JustKnit\Model\JustKnitProduct;
use JustKnit\Model\JustKnitOrder;
use JustKnit\Processors\Register\Base;

/**
 * Processor class which handles registration when Client with
 * posted [email] already exists
 *  - no new MODX user is created
 *  - active order, active product, subscriber_deletedafter,
 *    subscriber_deleted is updated, [JustKnitClientMeta]
 *  - active, primary_group updated [modUser]
 *  - phone, blocked, blockedafter, all address data updated [modUserProfile]
 *
 * @package justknit
 * @subpackage processors
 */
class UpdateClient extends Base
{
    /** @var modUser $user */
    public $user = null;

    /** @var modUserProfile $profile */
    public $profile = null;

    /** @var GoodNewsSubscriberMeta $subscribermeta */
    public $clientmeta = null;
    /**
     * @access public
     * @return mixed
     */
    public function process()
    {
        $this->user  = $this->controller->user;
        $this->profile = $this->controller->profile;
        $this->clientmeta = $this->controller->clientmeta;
        $this->order = $this->modx->newObject(JustKnitOrder::class);

        //Save order
        $this->setOrderFields();
        if (!$this->order->save()) {
            $this->modx->log(modX::LOG_LEVEL_ERROR, '[JustKnit] Could not save new order data - ' . $this->order->get('id') . ' with ds24_order_id: ' . $this->order->get('order_id'));
            $this->controller->setResponseMessage('[JustKnit] Could not save new order data - ' . $this->order->get('id') . ' with ds24_order_id: ' . $this->order->get('order_id'));
            return $this->modx->lexicon('justknit.user_err_save');
        }
        $this->controller->setProperty('active_order', $this->order->get('id'));
        $this->controller->setProperty('leave_group', $this->user->get('primary_group'));
        $this->setUserFields();
        if (!$this->user->save()) {
            $this->modx->log(modX::LOG_LEVEL_ERROR, '[JustKnit] Could not save client user data - ' . $this->user->get('id') . ' with username: ' . $this->user->get('username'));
            $this->controller->setResponseMessage('Could not save client user data ' . $this->user->get('id'));
            return $this->modx->lexicon('justknit.user_err_save');
        } else {
            $this->setUserGroup($this->user->get('primary_group'));
        }
        return true;
    }
    /**
     * Join user to new group and leave previous.
     *
     * @access public
     * @params int $group
     * @return void
     */
    public function setUserGroup(int $group) {
        // User must leave previous group
        // User can only be member of one group at the same time
        $leave = $this->controller->getProperty('leave_group');
        $this->user->leaveGroup($leave,'Member');
        $this->user->joinGroup($group,'Member');
        return true;
    }
    /**
     * Set the user data and create the user/profile objects.
     *
     * @access public
     * @return void
     */
    public function setUserFields() {
        $fields = $this->controller->getProperties();
//        $this->modx->log(modX::LOG_LEVEL_ERROR, '[JustKnit] $fields - unsetUserFields()' . print_r($fields,1));
        unset($fields['email']);
        unset($fields['address_id']);
        foreach ($fields as $k => $v) {
            $this->user->set($k, $v);
            $this->profile->set($k, $v);
            $this->clientmeta->set($k, $v);
        }
        $this->controller->setProperty('first_name', $fields['address_first_name']);
        $this->controller->setProperty('last_name', $fields['address_last_name']);
        return true;
    }

    /**
     * Set the Order data.
     *
     * @access public
     * @return void
     */
    public function setOrderFields()
    {
        $this->controller->setProperty('client_id' , $this->user->get('id'));
        $this->controller->setProperty('product_id' , $this->controller->getProperty('productID'));
        $this->order->fromArray($this->controller->getProperties());
    }
}
